# Copyright(c) 2022 Inspur
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.utils.translation import gettext_lazy as _

from horizon import tables


VMOVE_FILTER_CHOICES = (
    ('notification_uuid', _("Notification UUId ="), True),
    ('type', _("Type ="), True),
    ('status', _("Status ="), True),
)


class VMoveFilterAction(tables.FilterAction):
    filter_type = "server"
    filter_choices = VMOVE_FILTER_CHOICES


class VMoveTable(tables.DataTable):

    uuid = tables.Column('uuid', verbose_name=_("UUID"),
                         link="horizon:masakaridashboard:vmoves:detail")
    notification_id = tables.Column(
        'notification_id', verbose_name=_("Notification UUID"),
        link="horizon:masakaridashboard:notifications:detail")
    server_id = tables.Column(
        'server_id', verbose_name=_("Server ID"))
    server_name = tables.Column(
        'server_name', verbose_name=_("Server Name"))
    type = tables.Column('type', verbose_name=_("Type"))
    source_host = tables.Column(
        'source_host', verbose_name=_("Source Host"))
    dest_host = tables.Column(
        'dest_host', verbose_name=_("Dest Host"))
    start_time = tables.Column(
        'start_time', verbose_name=_("Start Time"))
    end_time = tables.Column(
        'end_time', verbose_name=_("End Time"))
    status = tables.Column(
        'status', verbose_name=_("Status"))

    def get_object_id(self, datum):
        return datum.notification_id + ',' + datum.uuid

    class Meta(object):
        name = "vmove"
        verbose_name = _("VMove")
        table_actions = (VMoveFilterAction,)



# Copyright (C) 2018 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools

from django.conf import settings
from django.utils.translation import gettext_lazy as _

from horizon.utils import functions as utils
from horizon.utils import memoized
from keystoneauth1.identity.generic import token
from keystoneauth1 import session as ks_session
from openstack import connection
from openstack_dashboard.api import nova as nova_api

from masakaridashboard.handle_errors import handle_errors


@memoized.memoized
def openstack_connection(request, version=None):
    interface = getattr(settings, 'OPENSTACK_ENDPOINT_TYPE', 'publicURL')

    auth = token.Token(
        auth_url=request.user.endpoint,
        token=request.user.token.id,
        project_name=request.user.project_name,
        project_id=request.user.tenant_id)
    cacert = getattr(settings, 'OPENSTACK_SSL_CACERT')
    session = ks_session.Session(auth=auth, verify=cacert or True)
    conn = connection.Connection(session=session,
                                 interface=interface,
                                 ha_api_version=version)

    return conn.instance_ha


def get_compute_service_list(request):
    return nova_api.service_list(request, binary="nova-compute")


@handle_errors(_("Unable to retrieve segments"), [])
def get_segment_list(request, marker='', paginate=False, filters=None):
    """Returns segments as per page size."""
    page_size = utils.get_page_size(request)
    client = openstack_connection(request)
    kwargs = get_request_param(marker, paginate, filters, page_size)
    entities_iter = client.segments(**kwargs)
    has_prev_data = has_more_data = False
    if paginate:
        entities, has_more_data, has_prev_data = pagination_process(
            entities_iter, kwargs['limit'], page_size, marker)
    else:
        entities = list(entities_iter)

    return entities, has_more_data, has_prev_data


def get_request_param(marker, paginate, filters, page_size):
    limit = getattr(settings, 'API_RESULT_LIMIT', 100)

    if paginate:
        request_size = page_size + 1
    else:
        request_size = limit

    kwargs = {"marker": marker,
              "limit": request_size
              }
    if filters is not None:
        kwargs.update(filters)
    return kwargs


def pagination_process(data, request_size, page_size, marker):
    """Retrieve a listing of specific entity and handles pagination.

    :param request: Request data
    :param marker: Pagination marker for large data sets: entity id
    :param paginate: If true will perform pagination based on settings.
                     Default:False
    """
    prev_data = more_data = False
    entities = list(itertools.islice(data, request_size))
    # first and middle page condition
    if len(entities) > page_size:
        entities.pop()
        more_data = True
        # middle page condition
        if marker is not None:
            prev_data = True
    elif marker is not None:
        prev_data = True

    return entities, more_data, prev_data


@handle_errors(_("Unable to retrieve segments"), [])
def segment_list(request):
    return list(openstack_connection(request).segments())


def segment_create(request, data):
    """Create segment."""
    return openstack_connection(request).create_segment(**data)


@handle_errors(_("Unable to retrieve segment"), [])
def get_segment(request, segment_id):
    """Returns segment by id"""
    return openstack_connection(request).get_segment(segment_id)


@handle_errors(_("Unable to delete segment"), [])
def segment_delete(request, segment_id, ignore_missing=True):
    return openstack_connection(request).delete_segment(
        segment_id, ignore_missing)


@handle_errors(_("Unable to update segment"), [])
def segment_update(request, segment_id, fields_to_update):
    """Update segment."""
    return openstack_connection(request).update_segment(
        segment_id, **fields_to_update)


def create_host(request, data):
    """Create Host."""
    attrs = {'name': data['name'],
             'reserved': data['reserved'],
             'type': data['type'],
             'control_attributes': data['control_attributes'],
             'on_maintenance': data['on_maintenance']}

    return openstack_connection(request).create_host(
        data['segment_id'], **attrs)


@handle_errors(_("Unable to get host list"), [])
def get_host_list(request, segment_id, filters):
    """Returns host list."""
    return openstack_connection(request).hosts(segment_id, **filters)


def delete_host(request, host_id, segment_id):
    return openstack_connection(request).delete_host(
        host_id, segment_id, False)


def update_host(request, host_uuid, failover_segment_id, fields_to_update):
    return openstack_connection(request).update_host(
        host_uuid, failover_segment_id, **fields_to_update)


def get_host(request, host_id, segment_id):
    """return single host """
    return openstack_connection(request).get_host(host_id, segment_id)


def notification_list(request, filters=None, marker='', paginate=False):
    """return notifications list """
    page_size = utils.get_page_size(request)
    kwargs = get_request_param(marker, paginate, filters, page_size)
    entities_iter = openstack_connection(request).notifications(**kwargs)
    has_prev_data = has_more_data = False
    if paginate:
        entities, has_more_data, has_prev_data = pagination_process(
            entities_iter, kwargs['limit'], page_size, marker)
    else:
        entities = list(entities_iter)

    return entities, has_more_data, has_prev_data


def get_notification_list(request):
    """return notifications list """
    return list(openstack_connection(request).notifications())


def get_notification(request, notification_id):
    """return single notifications"""
    return openstack_connection(request).get_notification(notification_id)


def get_notification_with_progress_details(request, notification_id):
    return openstack_connection(
        request, version='1.1').get_notification(
        notification_id)


@handle_errors(_("Unable to get vmoves list"), [])
def get_vmoves_list(request, notification_id, filters):
    """return vmoves list """
    return openstack_connection(
        request, version='1.3').vmoves(notification_id, **filters)


def get_vmove(request, notification_id, vmove_id):
    """return single vmove"""
    return openstack_connection(
        request, version='1.3').get_vmove(vmove_id, notification_id)



# Copyright(c) 2022 Inspur
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from horizon import exceptions
from horizon import tables
from horizon import tabs
from horizon.utils import memoized

from masakaridashboard.api import api
from masakaridashboard.vmoves import tables as masakari_tab
from masakaridashboard.vmoves import tabs as vmove_tab


class IndexView(tables.DataTableView):
    table_class = masakari_tab.VMoveTable
    template_name = 'masakaridashboard/vmoves/index.html'
    page_title = _("VMoves")

    def needs_filter_first(self, table):
        return self._needs_filter_first

    def get_data(self):
        notifications = api.get_notification_list(self.request)
        vmove_list = []
        filters = self.get_filters()
        self._needs_filter_first = True

        filter_first = getattr(settings, 'FILTER_DATA_FIRST', {})
        if filter_first.get('masakaridashboard.vmoves', False) and len(
                filters) == 0:
            self._needs_filter_first = True
            self._more = False
            return vmove_list

        for notification in notifications:
            if notification.type != "COMPUTE_HOST":
                continue
            vmove_gen = api.get_vmoves_list(
                self.request, notification.notification_uuid, filters)
            for item in vmove_gen:
                vmove_list.append(item)

        return vmove_list


class DetailView(tabs.TabbedTableView):
    tab_group_class = vmove_tab.VMoveDetailTabs
    template_name = 'horizon/common/_detail.html'
    page_title = "{{ vmove.server_name|default:vmove.server_name }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        vmove = self.get_data()
        table = masakari_tab.VMoveTable(self.request)
        context["vmove"] = vmove
        context["url"] = self.get_redirect_url()
        context["actions"] = table.render_row_actions(vmove)
        return context

    @memoized.memoized_method
    def get_data(self):
        try:
            row_data = self.kwargs['vmove_id'].split(',')
            notification_id = row_data[1]
            vmove_id = row_data[0]
            vmove = api.get_vmove(self.request, vmove_id, notification_id)
        except Exception:
            msg = _('Unable to get vmove "%s".') % vmove_id
            redirect = reverse('horizon:masakaridashboard:vmoves:index')
            exceptions.handle(self.request, msg, redirect=redirect)

        return vmove

    def get_redirect_url(self):
        return reverse('horizon:masakaridashboard:vmoves:index')

    def get_tabs(self, request, *args, **kwargs):
        vmove = self.get_data()
        return self.tab_group_class(request, vmove=vmove, **kwargs)



# Copyright (C) 2018 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from openstack_dashboard.test import helpers

from masakaridashboard.test.test_data import utils


class MasakariTestsMixin(object):
    def _setup_test_data(self):
        super(MasakariTestsMixin, self)._setup_test_data()
        utils.load_test_data(self)


class TestCase(MasakariTestsMixin, helpers.TestCase):

    def setUp(self):
        allowed_patch = mock.patch(
            "masakaridashboard.dashboard.MasakariDashboard.allowed",
            return_value=True)
        allowed_patch.start()
        self.addCleanup(mock.patch.stopall)
        super().setUp()



# Copyright (C) 2018 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import re_path

from masakaridashboard.segments import views


SEGMENT = r'^(?P<segment_id>[^/]+)/%s$'
urlpatterns = [
    re_path(r'^$', views.IndexView.as_view(), name='index'),
    re_path(r'^create_segment$', views.CreateSegmentView.as_view(),
            name='create_segment'),
    re_path(SEGMENT % 'detail', views.DetailView.as_view(), name='detail'),
    re_path(SEGMENT % 'update', views.UpdateView.as_view(), name='update'),
    re_path(SEGMENT % 'addhost', views.AddHostView.as_view(), name='addhost'),
]



# Copyright (C) 2018 NTT DATA
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from django.utils.translation import gettext_lazy as _

import horizon

from masakaridashboard.api import api
from masakaridashboard.default import panel

LOG = logging.getLogger(__name__)


class MasakariDashboard(horizon.Dashboard):
    slug = "masakaridashboard"
    name = _("Instance-ha")
    panels = ('default', 'segments', 'hosts', 'notifications', 'vmoves')
    default_panel = 'default'
    policy_rules = (('instance-ha', 'context_is_admin'),)

    def allowed(self, context):
        # disable whole dashboard if masakari
        # is not present in the service catalog
        try:
            # NOTE(pas-ha) this method tries to construct keystoneauth.Adapter
            # for the Instance-HA service,
            # which will fail if the service is absent
            api.openstack_connection(context['request'])
        except Exception as e:
            # catch all errors and log them,
            # no need to totally fail on e.g. HTTP connect failure
            LOG.warning(f"Failed to find suitable endpoint for Instance HA "
                        f"service, Masakari Dashboard will not be displayed. "
                        f"Error was: {e}")
            return False
        return super().allowed(context)


horizon.register(MasakariDashboard)
MasakariDashboard.register(panel.Default)



# Copyright(c) 2022 Inspur
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from django.urls import reverse

from masakaridashboard.test import helpers as test

INDEX_URL = reverse('horizon:masakaridashboard:vmoves:index')


class VMoveTest(test.TestCase):

    def test_index(self):
        vmoves = self.masakari_vmove.list()
        notifications = self.masakari_notification.list()
        with mock.patch('masakaridashboard.api.api.get_notification_list',
                        return_value=notifications), mock.patch(
                'masakaridashboard.api.api.get_notification',
                return_value=notifications[0]), mock.patch(
                'masakaridashboard.api.api.get_vmoves_list',
                return_value=vmoves):
            res = self.client.get(INDEX_URL)
        self.assertTemplateUsed(res, 'masakaridashboard/vmoves/index.html')
        self.assertEqual(res.status_code, 200)

    def test_detail(self):
        vmove = self.masakari_vmove.list()[0]
        id_to_update = vmove.uuid + ',' + vmove.notification_id
        detail_url = reverse('horizon:masakaridashboard:vmoves:detail',
                             args=[id_to_update])
        with mock.patch('masakaridashboard.api.api.get_vmove',
                        return_value=self.masakari_vmove.list()[0]):
            res = self.client.get(detail_url)
        self.assertNoFormErrors(res)
        self.assertEqual(200, res.status_code)
        self.assertTemplateUsed(res, 'horizon/common/_detail.html')
        self.assertTemplateUsed(
            res, 'masakaridashboard/vmoves/_detail_overview.html')
